@@ -80,6 +80,9 @@
 #define MAX_SEND_SGE		(1)
 #define MAX_RECV_SGE		(1)
 #define CTX_POLL_BATCH		(16)
+#define CTX_POLL_BATCH_INTENSE	(64)
+#define CQE_POLL_INTENSE_NUM_QPS_THRESHOLD		(2048)
+#define CQE_POLL_INTENSE_MSG_SIZE_THRESHOLD		(8192)
 #define PL			(1)
 #define ATOMIC_ADD_VALUE	(1)
 #define ATOMIC_SWAP_VALUE	(0)
@@ -99,15 +102,17 @@
 #define CLIENT_FD "/tmp/xrc_domain_client"
 #endif
 
+/* Vendors IDs */
+#define MLNX_VENDOR_ID 0x02c9
 
 #define NOTIFY_COMP_ERROR_SEND(wc,scnt,ccnt)                     											\
 	{ fprintf(stderr," Completion with error at client\n");      											\
-	  fprintf(stderr," Failed status %d: wr_id %d syndrom 0x%x\n",wc.status,(int) wc.wr_id,wc.vendor_err);	\
+	  fprintf(stderr," Failed status %d: qp_num %d wr_id %d syndrom 0x%x\n",wc.status,(int) wc.qp_num,(int) wc.wr_id,wc.vendor_err);	\
 	  fprintf(stderr, "scnt=%lu, ccnt=%lu\n",scnt, ccnt); }
 
 #define NOTIFY_COMP_ERROR_RECV(wc,rcnt)                     											    \
 	{ fprintf(stderr," Completion with error at server\n");      											\
-	  fprintf(stderr," Failed status %d: wr_id %d syndrom 0x%x\n",wc.status,(int) wc.wr_id,wc.vendor_err);	\
+	  fprintf(stderr," Failed status %d: qp_num %d wr_id %d syndrom 0x%x\n",wc.status,(int) wc.qp_num,(int) wc.wr_id,wc.vendor_err);	\
 	  fprintf(stderr," rcnt=%lu\n",rcnt); }
 
 /* Macro to determine packet size in case of UD. The UD addition is for the GRH . */
@@ -142,6 +147,42 @@
 		_a > _b ? _a : _b; \
 	})
 
+#define MIN(a, b) \
+	({ \
+		typeof(a) _a = (a); \
+		typeof(b) _b = (b); \
+		_a < _b ? _a : _b; \
+	})
+
+/*
+ * Unique WR ID is of the following format:
+ * |7        6|5        4|3               0|
+ * +----------+----------+-----------------+
+ * | Reserved | QP Index |    WR Index     |
+ * +----------+----------+-----------------+
+ * WR Index: Index of WR in perftest wr arrays.
+ * QP Index: Perftest QP index (not QP num) on which the WR is sent or received.
+ */
+#define WR_INDEX_OFFSET		0
+#define WR_INDEX_MASK		0xFFFFFFFF
+#define WR_ID_QP_INDEX_OFFSET	32
+#define WR_ID_QP_INDEX_MASK	0xFFFF
+
+static inline uint32_t get_wr_index(uint64_t wr_id)
+{
+	return wr_id & WR_INDEX_MASK;
+}
+
+static inline uint16_t get_wr_id_qp_index(uint64_t wr_id)
+{
+	return (wr_id >> WR_ID_QP_INDEX_OFFSET) & WR_ID_QP_INDEX_MASK;
+}
+
+static inline uint64_t build_wr_id(uint32_t wr_index, uint16_t qp_index)
+{
+	return ((uint64_t)wr_index) | ((uint64_t)qp_index << WR_ID_QP_INDEX_OFFSET);
+}
+
 /******************************************************************************
  * Perftest resources Structures and data types.
  ******************************************************************************/
@@ -165,6 +206,28 @@ struct cma {
 	int disconnects_left;
 };
 
+/* Dynamic CQE polling configuration */
+struct dyn_cqe_poll_config {
+	uint16_t min;
+	uint16_t max;
+	uint16_t stabilize;
+	double threshold;
+};
+
+/* Dynamic CQE polling state */
+struct dyn_poll_state {
+	uint16_t curr_size;
+	int stable_iters;
+	int last_ne;
+};
+
+/* Helper structure to hold dynamic polling context */
+struct dyn_poll_ctx {
+	struct dyn_cqe_poll_config config;
+	struct dyn_poll_state state;
+	int stabilization_iters;
+};
+
 struct pingpong_context {
 	struct cma cma_master;
 	struct rdma_event_channel		*cm_channel;
@@ -172,16 +235,19 @@ struct pingpong_context {
 	struct rdma_cm_id			*cm_id;
 	struct ibv_context			*context;
 	#ifdef HAVE_AES_XTS
-	struct mlx5dv_mkey			**mkey;
 	struct mlx5dv_dek			**dek;
 	#endif
+	#ifdef HAVE_SIG_OFFLOAD
+	struct mlx5dv_sig_block_domain *domain;
+	struct mlx5dv_sig_t10dif *t10dif_sig;
+	#endif
 	struct ibv_comp_channel			*recv_channel;
 	struct ibv_comp_channel			*send_channel;
 	struct ibv_pd				*pd;
 	#ifdef HAVE_TD_API
 	struct ibv_td				*td;
-	struct ibv_pd				*pad;
 	#endif
+	struct ibv_pd				*pad;
 	struct ibv_mr				**mr;
 	struct ibv_mr				*null_mr;
 	struct ibv_cq				*send_cq;
@@ -193,6 +259,7 @@ struct pingpong_context {
 	struct ibv_qp_ex			**qpx;
 	#ifdef HAVE_MLX5DV
 	struct mlx5dv_qp_ex			**dv_qp;
+	struct mlx5dv_mkey			**mkey;
 	#endif
 	int (*new_post_send_work_request_func_pointer) (struct pingpong_context *ctx, int index,
 		struct perftest_parameters *user_param);
@@ -213,7 +280,6 @@ struct pingpong_context {
 	int					tx_depth;
 	uint64_t				*scnt;
 	uint64_t				*ccnt;
-	int					is_contig_supported;
 	uint32_t				*r_dctn;
 	uint32_t				*dci_stream_id;
 	int 					dek_number;
@@ -232,6 +298,9 @@ struct pingpong_context {
 	int 					fd;
 	#endif
 	struct memory_ctx			*memory;
+	#ifdef HAVE_REG_MR_EX
+	struct ibv_dmah				*dmah;
+	#endif
 };
 
  struct pingpong_dest {
@@ -1019,4 +1088,6 @@ int rdma_cm_destroy_cma(struct pingpong_context *ctx,
 */
 int error_handler(char *error_message);
 
+void check_bf_support(struct pingpong_context *ctx);
+
 #endif /* PERFTEST_RESOURCES_H */
