@@ -187,6 +187,7 @@ int main(int argc, char *argv[])
 
 	/* Print basic test information. */
 	ctx_print_test_info(&user_param);
+	check_bf_support(&ctx);
 
 	/* shaking hands and gather the other side info. */
 	if (ctx_hand_shake(&user_comm,my_dest,rem_dest)) {
@@ -262,10 +263,12 @@ int main(int argc, char *argv[])
 			user_param.size = (uint64_t)1 << i;
 
 			if (user_param.verb == WRITE_IMM) {
-				/* Post receive recv_wqes fo current message size */
-				if (ctx_set_recv_wqes(&ctx,&user_param)) {
-					fprintf(stderr," Failed to post receive recv_wqes\n");
-					goto free_mem;
+				if (!user_param.use_unsolicited_write) {
+					/* Post receive recv_wqes fo current message size */
+					if (ctx_set_recv_wqes(&ctx,&user_param)) {
+						fprintf(stderr," Failed to post receive recv_wqes\n");
+						goto free_mem;
+					}
 				}
 
 				/* Sync between the client and server so the client won't send packets
